@@ -24,6 +24,19 @@ tmp4 = _prefix + "4"
 
 # routines to assist in molecular editing
 
+class undocontext:
+    def __init__(self, cmd, sele):
+        # not implemented in open-source
+        pass
+
+    def __enter__(self):
+        # not implemented in open-source
+        pass
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        # not implemented in open-source
+        pass
+
 def attach_fragment(selection,fragment,hydrogen,anchor,_self=cmd):
     '''
 ARGUMENTS
@@ -45,7 +58,7 @@ ARGUMENTS
             if _self.get_setting_boolean("auto_remove_hydrogens"):
                 _self.remove("(hydro and %s)"%fragment)
     else:
-        _self.fragment(fragment,tmp_editor)
+        _self.fragment(fragment,tmp_editor, origin=0)
         if _self.count_atoms("((%s) and elem H)"%selection,quiet=1):
             _self.fuse("(%s and id %d)"%(tmp_editor,hydrogen),"(pk1)",1)
             if _self.get_setting_boolean("auto_remove_hydrogens"):
@@ -55,20 +68,19 @@ ARGUMENTS
             _self.fuse("(%s and id %d)"%(tmp_editor,anchor),"(pk1)",1)
             if _self.get_setting_boolean("auto_remove_hydrogens"):
                 _self.remove("(hydro and pkmol)")            
+            elif _self.count_atoms('hydro and (neighbor pk2)'):
+                _self.h_fill()
         _self.delete(tmp_editor)
 
 def combine_fragment(selection,fragment,hydrogen,anchor,_self=cmd):
-    if selection in _self.get_names("selections"):
+    with undocontext(_self, selection):
         _self.fragment(fragment,tmp_editor)
-        if _self.count_atoms("((%s) and elem H)"%selection,quiet=1):
-            _self.fuse("(%s and id %d)"%(tmp_editor,hydrogen),"(pk1)",3)
+        try:
             if _self.get_setting_boolean("auto_remove_hydrogens"):
-                _self.remove("(hydro and pkmol)")            
-        else:
-            _self.fuse("(%s and id %d)"%(tmp_editor,anchor),"(pk1)",3)
-            if _self.get_setting_boolean("auto_remove_hydrogens"):
-                _self.remove("(hydro and pkmol)")            
-        _self.delete(tmp_editor)
+                _self.remove("(hydro and ?%s)" % tmp_editor)
+            _self.fuse("?%s" % tmp_editor, "(%s)" % selection, 3)
+        finally:
+            _self.delete(tmp_editor)
 
 #from time import time as ___time
 #___total = 0.0
@@ -151,7 +163,7 @@ ARGUMENTS
             else:
                 phi=180.0
                 psi=180.0
-        _self.fragment(amino_acid,tmp_editor)
+        _self.fragment(amino_acid,tmp_editor, origin=0)
         if _self.count_atoms("elem N",domain=tmp_connect):
             tmp = [ None ]
             _self.iterate(tmp_connect,"tmp[0]=resv", space={ 'tmp' : tmp })
@@ -368,7 +380,7 @@ def _fab(input,name,mode,resi,chain,segi,state,dir,hydro,ss,quiet,_self=cmd):
                     first = sequence.pop()
                     _self.fragment(code[first], tmp_obj)
                     if not hydro:
-                        cmd.remove(tmp_obj + ' and hydro')
+                        _self.remove(tmp_obj + ' and hydro')
                     _self.alter(tmp_obj,'resi="""%s""";chain="""%s""";segi="""%s"""'%(resi,chain,segi))
                     _self.create(name,tmp_obj+" or ?"+name,1,1,zoom=0)
                     tmp_sel = _self.get_unused_name()
@@ -403,7 +415,7 @@ def _fab(input,name,mode,resi,chain,segi,state,dir,hydro,ss,quiet,_self=cmd):
     return r
 
 def fab(input,name=None,mode='peptide',resi=1,chain='',segi='',state=-1,
-        dir=1,hydro=-1,ss=0,async=-1,quiet=1,_self=cmd):
+        dir=1,hydro=-1,ss=0,async_=0,quiet=1,_self=cmd, **kwargs):
     '''
 DESCRIPTION
 
@@ -422,7 +434,12 @@ EXAMPLE
     fab ACDEFGH
     fab ACDEFGH, helix, ss=1
     '''
-    if async<1:
+    async_ = int(kwargs.pop('async', async_))
+
+    if kwargs:
+        raise pymol.CmdException('unknown argument: ' + ', '.join(kwargs))
+
+    if async_ < 1:
         r = _fab(input,name,mode,resi,chain,segi,
                  state,dir,hydro,ss,quiet,_self)
     else:
